﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.IO;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace arduino
{
    public partial class oscill : Form
    {
        int oscillPoints = 0;
        double oscillHigest = 0;
        string oscillName = "";
        string fileName;
        StreamReader streamReader;

        public oscill()
        {
            InitializeComponent();
            nudYMin.Value = Math.Max(0,Properties.Settings.Default.nudYMin);
            nudYMax.Value = Math.Max(nudYMin.Value+1, Properties.Settings.Default.nudYMax);
            tbMultPause.Text = Properties.Settings.Default.tbMultPause;
        }
        public void initOscill( string header, string fn ){
            string[] s = header.Split(new Char[] { ',' });
            int j, k, l; double d;

            if (s.Length == 7)
            {
                fileName = fn;
                oscillPoints = Convert.ToInt16(s[1]);
                try
                {
                    d = Convert.ToDouble(s[2].Replace(".", ","));
                }
                catch
                {
                    d = Convert.ToDouble(s[2]);
                }
                oscillHigest = d;

                oscillName = s[3];
                j = Convert.ToInt16(s[4]);
                k = Convert.ToInt16(s[5]);
                l = Convert.ToInt16(s[6]);
                
                chart1.Series[0].Points.Clear();
                chart1.Series[0].LegendText = oscillName;
                chart1.Series[0].Color = Color.FromArgb(j, k, l);

            }
        }
        public void deInitOscill()
        {
            oscillPoints = 0;
            oscillHigest = 0;
            oscillName = "";
            multStop();
            fileName = "";
        }
        public void showData(string data)
        {
            string[] p = data.Split(new Char[] { ':' });
            if (p.Length < 3) return;
            if (p[1] != "o") return;
            string date = p[0].Replace("|", ":"); // дата строки
            try
            {
                chart1.Series[0].Points.Clear();
                lbTimestamp.Text = date;
                for (int i = 0; i < oscillPoints; i++)
                {
                    double t = Convert.ToDouble(p[2]) / oscillPoints;
                    t = Math.Truncate(t * 100) / 100;
                    double value = Convert.ToDouble(p[i + 3]) / 1023 * oscillHigest;
                    value = Math.Truncate(value * 1000) / 1000;
                    chart1.Series[0].LegendText = value.ToString();
                    chart1.Series[0].Points.AddXY(i * t, value);
                }
//                chart1.ResumeLayout();
            }
            catch { };
        }

        
        private void oscill_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (sender == this)
            {
                e.Cancel = true;
                multStop();
                this.WindowState = FormWindowState.Minimized;
            }
        }


        private void btnSaveImage_Click(object sender, EventArgs e)
        {
            // Create a new save file dialog
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();

            // Sets the current file name filter string, which determines 
            // the choices that appear in the "Save as file type" or 
            // "Files of type" box in the dialog box.
            saveFileDialog1.Filter = "Bitmap (*.bmp)|*.bmp|JPEG (*.jpg)|*.jpg|PNG (*.png)|*.png|GIF (*.gif)|*.gif";
            saveFileDialog1.FilterIndex = 3;
            saveFileDialog1.RestoreDirectory = true;
            saveFileDialog1.FileName = "oscill-"+DateTime.Now.ToString("yy.MM.dd HH.mm.ss");

            // Set image file format
            if (saveFileDialog1.ShowDialog() == DialogResult.OK)
            {
                ChartImageFormat format = ChartImageFormat.Bmp;

                if (saveFileDialog1.FileName.EndsWith("bmp"))
                {
                    format = ChartImageFormat.Bmp;
                }
                else if (saveFileDialog1.FileName.EndsWith("jpg"))
                {
                    format = ChartImageFormat.Jpeg;
                }
                else if (saveFileDialog1.FileName.EndsWith("gif"))
                {
                    format = ChartImageFormat.Gif;
                }
                else if (saveFileDialog1.FileName.EndsWith("png"))
                {
                    format = ChartImageFormat.Png;
                }

                // Save image
                chart1.SaveImage(saveFileDialog1.FileName, format);
            }
        }

        private void btnSaveAllImages_Click(object sender, EventArgs e)
        {
            if (fileName != "")
            {
                int newInterval;
                try
                {
                    newInterval = Convert.ToInt16(tbMultPause.Text);
                }
                catch {
                    newInterval = tmMult.Interval;
                    tbMultPause.Text = newInterval.ToString();
                }
                if (tmMult.Interval != newInterval)
                {
                    tmMult.Interval = newInterval;
                    Properties.Settings.Default.tbMultPause = tmMult.Interval.ToString();
                    Properties.Settings.Default.Save();
                }

                if (tmMult.Enabled == false)
                {
                    streamReader = new StreamReader(fileName);
                    tmMult.Enabled = true;
                    tbMultPause.Enabled = false;
                }
                else {
                    multStop();
                }
            }
        }
        private void multStop() {
            tmMult.Enabled = false;
            if (streamReader != null) 
                try { streamReader.Close(); }
                catch { }
            lbTimestamp.Text = "";
            tbMultPause.Enabled = true;
        }
        private void tmMult_Tick(object sender, EventArgs e)
        {
            if (streamReader != null)
            { 
                string inp;
                while (!streamReader.EndOfStream)
                {
                    inp = streamReader.ReadLine();
                    if (!inp.StartsWith("#"))
                    {
                        showData(inp);
                        break;
                    }
                }
                if (streamReader.EndOfStream) multStop();
            }
        }

        private void nudYMin_ValueChanged(object sender, EventArgs e)
        {
            chart1.ChartAreas[0].AxisY.Minimum = Math.Min( chart1.ChartAreas[0].AxisY.Maximum-1, (double)nudYMin.Value);
            Properties.Settings.Default.nudYMin = (int)chart1.ChartAreas[0].AxisY.Minimum;
            Properties.Settings.Default.Save();
        }

        private void nudYMax_ValueChanged(object sender, EventArgs e)
        {
            chart1.ChartAreas[0].AxisY.Maximum = Math.Max(chart1.ChartAreas[0].AxisY.Minimum + 1, (double)nudYMax.Value);
            Properties.Settings.Default.nudYMax = (int)chart1.ChartAreas[0].AxisY.Maximum;
            Properties.Settings.Default.Save();
        }
    }
}
